Module: Aspera::Cli::Allowed

Defined in:
lib/aspera/cli/manager.rb

Overview

Constants to be used as parameter ‘allowed:` for `OptionValue`

Constant Summary collapse

TYPES_STRING_ARRAY =

This option can be set to a single string or array, multiple times, and gives Array of String

[Array, String].freeze
TYPES_SYMBOL_ARRAY =

A list of symbols with constrained values

[Array, Symbol].freeze
TYPES_INTEGER =

Value will be coerced to int

[Integer].freeze
TYPES_BOOLEAN =
BoolValue::TYPES
TYPES_NONE =

No value at all for the option, it’s a switch, like ‘-N`

[].freeze
TYPES_ENUM =

Symbol

[Symbol].freeze
TYPES_STRING =

String

[String].freeze