Class: Aspera::Ssh::Error
- Inherits:
-
Error
- Object
- Error
- Aspera::Ssh::Error
- Defined in:
- lib/aspera/ssh.rb
Overview
Raised when the remote command fails or cannot be started
Raised when the remote command fails or cannot be started