Class: Aspera::Cli::Manager
- Inherits:
-
Object
- Object
- Aspera::Cli::Manager
- Defined in:
- lib/aspera/cli/manager.rb
Overview
parse command line options arguments options start with ‘-’, others are commands resolves on extended value syntax
Constant Summary collapse
- BOOLEAN_SIMPLE =
%i[no yes].freeze
Instance Attribute Summary collapse
-
#ask_missing_mandatory ⇒ Object
Returns the value of attribute ask_missing_mandatory.
-
#ask_missing_optional ⇒ Object
Returns the value of attribute ask_missing_optional.
-
#fail_on_missing_mandatory ⇒ Object
writeonly
Sets the attribute fail_on_missing_mandatory.
-
#parser ⇒ Object
readonly
Returns the value of attribute parser.
Class Method Summary collapse
-
.enum_to_bool(enum) ⇒ Object
‘true` if value is a value for `true` in BOOLEAN_VALUES.
-
.enum_to_yes_no(enum) ⇒ Object
:yes ot :no.
-
.get_from_list(short_value, descr, allowed_values) ⇒ Object
Find shortened string value in allowed symbol list.
-
.multi_choice_assert_msg(error_msg, accept_list) ⇒ Object
Generates error message with list of allowed values.
-
.option_line_to_name(name) ⇒ Object
change option name with dash to name with underscore.
- .option_name_to_line(name) ⇒ Object
Instance Method Summary collapse
-
#add_option_preset(preset_hash, where, override: true) ⇒ Object
Adds each of the keys of specified hash as an option.
-
#args_as_extended(value) ⇒ Object
Read remaining args and build an Array or Hash.
-
#clear_option(option_symbol) ⇒ Object
Set option to ‘nil`.
-
#command_or_arg_empty? ⇒ Boolean
Check if there were unprocessed values to generate error.
-
#declare(option_symbol, description, short: nil, allowed: nil, default: nil, handler: nil, deprecation: nil, &block) ⇒ Object
Declare an option.
-
#final_errors ⇒ Object
Unprocessed options or arguments ?.
-
#get_interactive(descr, check_option: false, multiple: false, accept_list: nil) ⇒ Object
Prompt user for input in a list of symbols.
-
#get_next_argument(descr, mandatory: true, multiple: false, accept_list: nil, validation: Allowed::TYPES_STRING, aliases: nil, default: nil) ⇒ Object
One value, list or nil (if optional and no default).
- #get_next_command(command_list, aliases: nil) ⇒ Object
-
#get_option(option_symbol, mandatory: false) ⇒ Object
Get an option value by name either return value or calls handler, can return nil ask interactively if requested/required.
-
#initialize(program_name, argv = nil) ⇒ Manager
constructor
A new instance of Manager.
-
#known_options(only_defined: false) ⇒ Hash
Options as taken from config file and command line just before command execution.
-
#parse_options! ⇒ Object
Removes already known options from the list.
- #prompt_user_input(prompt, sensitive: false) ⇒ Object
-
#prompt_user_input_in_list(prompt, sym_list) ⇒ Symbol
prompt user for input in a list of symbols.
-
#set_option(option_symbol, value, where: 'code override') ⇒ Object
Set an option value by name, either store value or call handler String is given to extended value.
-
#unprocessed_options_with_value ⇒ Hash
Get all original options on command line used to generate a config in config file.
-
#unshift_next_argument(argument) ⇒ Object
Allows a plugin to add an argument as next argument to process.
Constructor Details
#initialize(program_name, argv = nil) ⇒ Manager
Returns a new instance of Manager.
191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 |
# File 'lib/aspera/cli/manager.rb', line 191 def initialize(program_name, argv = nil) # command line values *not* starting with '-' @unprocessed_cmd_line_arguments = [] # command line values starting with '-' @unprocessed_cmd_line_options = [] # a copy of all initial options @initial_cli_options = [] # option description: option_symbol => OptionValue @declared_options = {} # do we ask missing options and arguments to user ? @ask_missing_mandatory = false # STDIN.isatty # ask optional options if not provided and in interactive @ask_missing_optional = false # get_option fails if a mandatory parameter is asked @fail_on_missing_mandatory = true # Array of [key(sym), value] # those must be set before parse # parse consumes those defined only @option_pairs_batch = {} @option_pairs_env = {} # NOTE: was initially inherited but it is preferred to have specific methods @parser = OptionParser.new @parser.program_name = program_name # options can also be provided by env vars : --param-name -> ASCLI_PARAM_NAME env_prefix = program_name.upcase + OPTION_SEP_SYMBOL ENV.each do |k, v| @option_pairs_env[k[env_prefix.length..-1].downcase.to_sym] = v if k.start_with?(env_prefix) end Log.log.debug{"env=#{@option_pairs_env}".red} @unprocessed_cmd_line_options = [] @unprocessed_cmd_line_arguments = [] return if argv.nil? # true until `--` is found (stop options) = true until argv.empty? value = argv.shift if && value.start_with?('-') Log.log.trace1{"opt: #{value}"} if value.eql?(OPTIONS_STOP) = false else @unprocessed_cmd_line_options.push(value) end else Log.log.trace1{"arg: #{value}"} @unprocessed_cmd_line_arguments.push(value) end end @initial_cli_options = @unprocessed_cmd_line_options.dup.freeze Log.log.trace1{"add_cmd_line_options:commands/arguments=#{@unprocessed_cmd_line_arguments},options=#{@unprocessed_cmd_line_options}".red} @parser.separator('') @parser.separator('OPTIONS: global') declare(:interactive, 'Use interactive input of missing params', allowed: Allowed::TYPES_BOOLEAN, handler: {o: self, m: :ask_missing_mandatory}) declare(:ask_options, 'Ask even optional options', allowed: Allowed::TYPES_BOOLEAN, handler: {o: self, m: :ask_missing_optional}) # do not parse options yet, let's wait for option `-h` to be overridden end |
Instance Attribute Details
#ask_missing_mandatory ⇒ Object
Returns the value of attribute ask_missing_mandatory.
188 189 190 |
# File 'lib/aspera/cli/manager.rb', line 188 def ask_missing_mandatory @ask_missing_mandatory end |
#ask_missing_optional ⇒ Object
Returns the value of attribute ask_missing_optional.
188 189 190 |
# File 'lib/aspera/cli/manager.rb', line 188 def ask_missing_optional @ask_missing_optional end |
#fail_on_missing_mandatory=(value) ⇒ Object (writeonly)
Sets the attribute fail_on_missing_mandatory
189 190 191 |
# File 'lib/aspera/cli/manager.rb', line 189 def fail_on_missing_mandatory=(value) @fail_on_missing_mandatory = value end |
#parser ⇒ Object (readonly)
Returns the value of attribute parser.
187 188 189 |
# File 'lib/aspera/cli/manager.rb', line 187 def parser @parser end |
Class Method Details
.enum_to_bool(enum) ⇒ Object
Returns ‘true` if value is a value for `true` in BOOLEAN_VALUES.
146 147 148 149 |
# File 'lib/aspera/cli/manager.rb', line 146 def enum_to_bool(enum) Aspera.assert_values(enum, BOOLEAN_VALUES){'boolean'} return TRUE_VALUES.include?(enum) end |
.enum_to_yes_no(enum) ⇒ Object
Returns :yes ot :no.
152 153 154 155 |
# File 'lib/aspera/cli/manager.rb', line 152 def enum_to_yes_no(enum) Aspera.assert_values(enum, BOOLEAN_VALUES){'boolean'} return TRUE_VALUES.include?(enum) ? BOOL_YES : BOOL_NO end |
.get_from_list(short_value, descr, allowed_values) ⇒ Object
Find shortened string value in allowed symbol list
158 159 160 161 162 163 164 165 166 167 168 |
# File 'lib/aspera/cli/manager.rb', line 158 def get_from_list(short_value, descr, allowed_values) Aspera.assert_type(short_value, String) # we accept shortcuts matching_exact = allowed_values.select{ |i| i.to_s.eql?(short_value)} return matching_exact.first if matching_exact.length == 1 matching = allowed_values.select{ |i| i.to_s.start_with?(short_value)} Aspera.assert(!matching.empty?, multi_choice_assert_msg("unknown value for #{descr}: #{short_value}", allowed_values), type: BadArgument) Aspera.assert(matching.length.eql?(1), multi_choice_assert_msg("ambiguous shortcut for #{descr}: #{short_value}", matching), type: BadArgument) return enum_to_bool(matching.first) if allowed_values.eql?(BOOLEAN_VALUES) return matching.first end |
.multi_choice_assert_msg(error_msg, accept_list) ⇒ Object
Generates error message with list of allowed values
173 174 175 |
# File 'lib/aspera/cli/manager.rb', line 173 def multi_choice_assert_msg(error_msg, accept_list) [error_msg, 'Use:'].concat(accept_list.map{ |c| "- #{c}"}.sort).join("\n") end |
.option_line_to_name(name) ⇒ Object
change option name with dash to name with underscore
178 179 180 |
# File 'lib/aspera/cli/manager.rb', line 178 def option_line_to_name(name) return name.gsub(OPTION_SEP_LINE, OPTION_SEP_SYMBOL) end |
.option_name_to_line(name) ⇒ Object
182 183 184 |
# File 'lib/aspera/cli/manager.rb', line 182 def option_name_to_line(name) return "#{OPTION_PREFIX}#{name.to_s.gsub(OPTION_SEP_SYMBOL, OPTION_SEP_LINE)}" end |
Instance Method Details
#add_option_preset(preset_hash, where, override: true) ⇒ Object
Adds each of the keys of specified hash as an option
409 410 411 412 413 414 415 416 |
# File 'lib/aspera/cli/manager.rb', line 409 def add_option_preset(preset_hash, where, override: true) Aspera.assert_type(preset_hash, Hash) Log.log.debug{"add_option_preset: #{preset_hash}, #{where}, #{override}"} preset_hash.each do |k, v| option_symbol = k.to_sym @option_pairs_batch[option_symbol] = v if override || !@option_pairs_batch.key?(option_symbol) end end |
#args_as_extended(value) ⇒ Object
Read remaining args and build an Array or Hash
564 565 566 567 568 569 570 571 572 573 574 |
# File 'lib/aspera/cli/manager.rb', line 564 def args_as_extended(value) # This extended value does not take args (`@:`) ExtendedValue.assert_no_value(value, :p) result = nil get_next_argument(:args, multiple: true).each do |arg| Aspera.assert(arg.include?(OPTION_VALUE_SEPARATOR)){"Positional argument: #{arg} does not inlude #{OPTION_VALUE_SEPARATOR}"} path, raw = arg.split(OPTION_VALUE_SEPARATOR, 2) result = dotted_to_extended(path, raw, result) end result end |
#clear_option(option_symbol) ⇒ Object
Set option to ‘nil`
399 400 401 402 403 |
# File 'lib/aspera/cli/manager.rb', line 399 def clear_option(option_symbol) Aspera.assert_type(option_symbol, Symbol) Aspera.assert(@declared_options.key?(option_symbol), type: Cli::BadArgument){"Unknown option: #{option_symbol}"} @declared_options[option_symbol].clear end |
#command_or_arg_empty? ⇒ Boolean
Check if there were unprocessed values to generate error
424 425 426 |
# File 'lib/aspera/cli/manager.rb', line 424 def command_or_arg_empty? return @unprocessed_cmd_line_arguments.empty? end |
#declare(option_symbol, description, short: nil, allowed: nil, default: nil, handler: nil, deprecation: nil, &block) ⇒ Object
Declare an option
257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 |
# File 'lib/aspera/cli/manager.rb', line 257 def declare(option_symbol, description, short: nil, allowed: nil, default: nil, handler: nil, deprecation: nil, &block) Aspera.assert_type(option_symbol, Symbol) Aspera.assert(!@declared_options.key?(option_symbol)){"#{option_symbol} already declared"} Aspera.assert(description[-1] != '.'){"#{option_symbol} ends with dot"} Aspera.assert(description[0] == description[0].upcase){"#{option_symbol} description does not start with an uppercase"} Aspera.assert(!['hash', 'extended value'].any?{ |s| description.downcase.include?(s)}){"#{option_symbol} shall use :allowed"} Aspera.assert_type(handler, Hash) if handler Aspera.assert(handler.keys.sort.eql?(%i[m o])) if handler option_attrs = @declared_options[option_symbol] = OptionValue.new( option: option_symbol, description: description, allowed: allowed, handler: handler, deprecation: deprecation ) real_types = option_attrs.types&.reject{ |i| [NilClass, String, Symbol].include?(i)} description = "#{description} (#{real_types.map(&:name).join(', ')})" if real_types && !real_types.empty? && !real_types.eql?(Allowed::TYPES_ENUM) && !real_types.eql?(Allowed::TYPES_BOOLEAN) && !real_types.eql?(Allowed::TYPES_STRING) description = "#{description} (#{'deprecated'.blue}: #{deprecation})" if deprecation set_option(option_symbol, default, where: 'default') unless default.nil? on_args = [description] case option_attrs.types when Allowed::TYPES_ENUM, Allowed::TYPES_BOOLEAN # This option value must be a symbol (or array of symbols) set_option(option_symbol, Manager.enum_to_bool(default), where: 'default') if option_attrs.values.eql?(BOOLEAN_VALUES) && !default.nil? value = get_option(option_symbol) help_values = if option_attrs.types.eql?(Allowed::TYPES_BOOLEAN) highlight_current_in_list(BOOLEAN_SIMPLE, self.class.enum_to_yes_no(value)) else highlight_current_in_list(option_attrs.values, value) end on_args[0] = "#{description}: #{help_values}" on_args.push(symbol_to_option(option_symbol, 'ENUM')) # on_args.push(option_attrs.values) @parser.on(*on_args) do |v| set_option(option_symbol, self.class.get_from_list(v.to_s, description, option_attrs.values), where: SOURCE_USER) end when Allowed::TYPES_NONE Aspera.assert_type(block, Proc){"missing execution block for #{option_symbol}"} on_args.push(symbol_to_option(option_symbol)) on_args.push("-#{short}") if short.is_a?(String) @parser.on(*on_args, &block) else on_args.push(symbol_to_option(option_symbol, 'VALUE')) on_args.push("-#{short}VALUE") unless short.nil? # coerce integer on_args.push(Integer) if option_attrs.types.eql?(Allowed::TYPES_INTEGER) @parser.on(*on_args) do |v| set_option(option_symbol, v, where: SOURCE_USER) end end Log.log.trace1{"on_args=#{on_args}"} end |
#final_errors ⇒ Object
Unprocessed options or arguments ?
429 430 431 432 433 434 |
# File 'lib/aspera/cli/manager.rb', line 429 def final_errors result = [] result.push("unprocessed options: #{@unprocessed_cmd_line_options}") unless @unprocessed_cmd_line_options.empty? result.push("unprocessed values: #{@unprocessed_cmd_line_arguments}") unless @unprocessed_cmd_line_arguments.empty? return result end |
#get_interactive(descr, check_option: false, multiple: false, accept_list: nil) ⇒ Object
Prompt user for input in a list of symbols
534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 |
# File 'lib/aspera/cli/manager.rb', line 534 def get_interactive(descr, check_option: false, multiple: false, accept_list: nil) option_attrs = @declared_options[descr.to_sym] what = option_attrs ? 'option' : 'argument' if !@ask_missing_mandatory = "missing #{what}: #{descr}" if accept_list.nil? raise Cli::BadArgument, else Aspera.assert(false, self.class.multi_choice_assert_msg(, accept_list), type: Cli::MissingArgument) end end default_prompt = "#{what}: #{descr}" # ask interactively result = [] puts(' (one per line, end with empty line)') if multiple loop do prompt = default_prompt prompt = "#{accept_list.join(' ')}\n#{default_prompt}" if accept_list entry = prompt_user_input(prompt, sensitive: option_attrs&.sensitive) break if entry.empty? && multiple entry = ExtendedValue.instance.evaluate(entry, context: 'interactive input') entry = self.class.get_from_list(entry, descr, accept_list) if accept_list return entry unless multiple result.push(entry) end return result end |
#get_next_argument(descr, mandatory: true, multiple: false, accept_list: nil, validation: Allowed::TYPES_STRING, aliases: nil, default: nil) ⇒ Object
Returns one value, list or nil (if optional and no default).
319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 |
# File 'lib/aspera/cli/manager.rb', line 319 def get_next_argument(descr, mandatory: true, multiple: false, accept_list: nil, validation: Allowed::TYPES_STRING, aliases: nil, default: nil) Aspera.assert_array_all(accept_list, Symbol) unless accept_list.nil? Aspera.assert_hash_all(aliases, Symbol, Symbol) unless aliases.nil? validation = Symbol unless accept_list.nil? validation = [validation] unless validation.is_a?(Array) || validation.nil? Aspera.assert_array_all(validation, Class){'validation'} unless validation.nil? descr = "#{descr} (#{validation.join(', ')})" unless validation.nil? || validation.eql?(Allowed::TYPES_STRING) result = if !@unprocessed_cmd_line_arguments.empty? how_many = multiple ? @unprocessed_cmd_line_arguments.length : 1 values = @unprocessed_cmd_line_arguments.shift(how_many) values = values.map{ |v| ExtendedValue.instance.evaluate(v, context: "argument: #{descr}", allowed: validation)} # if expecting list and only one arg of type array : it is the list values = values.first if multiple && values.length.eql?(1) && values.first.is_a?(Array) if accept_list allowed_values = [].concat(accept_list) allowed_values.concat(aliases.keys) unless aliases.nil? values = values.map{ |v| self.class.get_from_list(v, descr, allowed_values)} end multiple ? values : values.first elsif !default.nil? then default # no value provided, either get value interactively, or exception elsif mandatory then get_interactive(descr, multiple: multiple, accept_list: accept_list) end if result.is_a?(String) && validation&.eql?(Allowed::TYPES_INTEGER) int_result = Integer(result, exception: false) raise Cli::BadArgument, "Invalid integer: #{result}" if int_result.nil? result = int_result end Log.log.debug{"#{descr}=#{result}"} result = aliases[result] if aliases&.key?(result) # if value comes from JSON/YAML, it may come as Integer result = result.to_s if result.is_a?(Integer) && validation&.eql?(Allowed::TYPES_STRING) if validation && (mandatory || !result.nil?) value_list = multiple ? result : [result] value_list.each do |value| raise Cli::BadArgument, "Argument #{descr} is a #{value.class} but must be #{'one of: ' if validation.length > 1}#{validation.map(&:name).join(', ')}" unless validation.any?{ |t| value.is_a?(t)} end end return result end |
#get_next_command(command_list, aliases: nil) ⇒ Object
362 |
# File 'lib/aspera/cli/manager.rb', line 362 def get_next_command(command_list, aliases: nil); return get_next_argument('command', accept_list: command_list, aliases: aliases); end |
#get_option(option_symbol, mandatory: false) ⇒ Object
Get an option value by name either return value or calls handler, can return nil ask interactively if requested/required
368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 |
# File 'lib/aspera/cli/manager.rb', line 368 def get_option(option_symbol, mandatory: false) Aspera.assert_type(option_symbol, Symbol) Aspera.assert(@declared_options.key?(option_symbol), type: Cli::BadArgument){"Unknown option: #{option_symbol}"} option_attrs = @declared_options[option_symbol] result = option_attrs.value # Do not fail for manual generation if option mandatory but not set return :skip_missing_mandatory if result.nil? && mandatory && !@fail_on_missing_mandatory if result.nil? if !@ask_missing_mandatory Aspera.assert(!mandatory, type: Cli::BadArgument){"Missing mandatory option: #{option_symbol}"} elsif @ask_missing_optional || mandatory # ask_missing_mandatory result = get_interactive(option_symbol.to_s, check_option: true, accept_list: option_attrs.values) set_option(option_symbol, result, where: 'interactive') end end return result end |
#known_options(only_defined: false) ⇒ Hash
Returns options as taken from config file and command line just before command execution.
461 462 463 464 465 466 467 468 469 470 |
# File 'lib/aspera/cli/manager.rb', line 461 def (only_defined: false) result = {} @declared_options.each_key do |option_symbol| v = get_option(option_symbol) result[option_symbol] = v unless only_defined && v.nil? rescue => e result[option_symbol] = e.to_s end return result end |
#parse_options! ⇒ Object
Removes already known options from the list
473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 |
# File 'lib/aspera/cli/manager.rb', line 473 def Log.log.trace1('parse_options!'.red) # First options from conf file consume_option_pairs(@option_pairs_batch, 'set') # Then, env var (to override) consume_option_pairs(@option_pairs_env, 'env') # Then, command line override = [] begin # remove known options one by one, exception if unknown Log.log.trace1('Before parse') Log.dump(:unprocessed_cmd_line_options, @unprocessed_cmd_line_options) @parser.parse!(@unprocessed_cmd_line_options) Log.log.trace1('After parse') rescue OptionParser::InvalidOption => e Log.log.trace1{"InvalidOption #{e}".red} if (m = e.args.first.match(/^--([a-z\-]+)\.([^=]+)=(.+)$/)) option, path, value = m.captures option_sym = self.class.option_line_to_name(option).to_sym if @declared_options.key?(option_sym) set_option(option_sym, dotted_to_extended(path, value), where: 'dotted') retry end end # save for later processing .push(e.args.first) retry end Log.log.trace1{"remains: #{}"} # set unprocessed options for next time @unprocessed_cmd_line_options = end |
#prompt_user_input(prompt, sensitive: false) ⇒ Object
506 507 508 509 510 511 512 |
# File 'lib/aspera/cli/manager.rb', line 506 def prompt_user_input(prompt, sensitive: false) return $stdin.getpass("#{prompt}> ") if sensitive print("#{prompt}> ") line = $stdin.gets Aspera.assert_type(line, String){'Unexpected end of standard input'} return line.chomp end |
#prompt_user_input_in_list(prompt, sym_list) ⇒ Symbol
prompt user for input in a list of symbols
518 519 520 521 522 523 524 525 526 527 |
# File 'lib/aspera/cli/manager.rb', line 518 def prompt_user_input_in_list(prompt, sym_list) loop do input = prompt_user_input(prompt).to_sym if sym_list.any?{ |a| a.eql?(input)} return input else $stderr.puts("No such #{prompt}: #{input}, select one of: #{sym_list.join(', ')}") # rubocop:disable Style/StderrPuts end end end |
#set_option(option_symbol, value, where: 'code override') ⇒ Object
Set an option value by name, either store value or call handler String is given to extended value
392 393 394 395 396 |
# File 'lib/aspera/cli/manager.rb', line 392 def set_option(option_symbol, value, where: 'code override') Aspera.assert_type(option_symbol, Symbol) Aspera.assert(@declared_options.key?(option_symbol), type: Cli::BadArgument){"Unknown option: #{option_symbol}"} @declared_options[option_symbol].assign_value(value, where: where) end |
#unprocessed_options_with_value ⇒ Hash
Get all original options on command line used to generate a config in config file
438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 |
# File 'lib/aspera/cli/manager.rb', line 438 def result = {} @initial_cli_options.each do |option_value| case option_value when /^#{OPTION_PREFIX}([^=]+)$/o # ignore when /^#{OPTION_PREFIX}([^=]+)=(.*)$/o name = Regexp.last_match(1) value = Regexp.last_match(2) name.gsub!(OPTION_SEP_LINE, OPTION_SEP_SYMBOL) value = ExtendedValue.instance.evaluate(value, context: "option: #{name}") Log.log.debug{"option #{name}=#{value}"} result[name] = value @unprocessed_cmd_line_options.delete(option_value) else raise Cli::BadArgument, "wrong option format: #{option_value}" end end return result end |
#unshift_next_argument(argument) ⇒ Object
Allows a plugin to add an argument as next argument to process
419 420 421 |
# File 'lib/aspera/cli/manager.rb', line 419 def unshift_next_argument(argument) @unprocessed_cmd_line_arguments.unshift(argument) end |