Class: Aspera::Cli::Plugins::Base
- Inherits:
-
Object
- Object
- Aspera::Cli::Plugins::Base
- Defined in:
- lib/aspera/cli/plugins/base.rb
Overview
Base class for command plugins
Defined Under Namespace
Modules: Operations
Instance Attribute Summary collapse
-
#context ⇒ Object
readonly
Global objects.
Class Method Summary collapse
Instance Method Summary collapse
- #add_manual_header(has_options = true) ⇒ Object
- #config ⇒ Aspera::Cli::Plugins::Config
-
#do_bulk_operation(command:, descr: nil, values: Hash, id_result: 'id', fields: :default, schema: nil) {|param| ... } ⇒ Hash
For create and delete operations: execute one action or multiple if bulk is yes.
-
#entity_execute(api:, entity:, command: nil, display_fields: nil, items_key: nil, delete_style: nil, id_as_arg: false, is_singleton: false, list_query: nil) {|value| ... } ⇒ Hash
Operations: Create, Delete, Show, List, Modify.
- #formatter ⇒ Aspera::Cli::Formatter
-
#initialize(context:) ⇒ Base
constructor
A new instance of Base.
- #options ⇒ Aspera::Cli::Manager
- #persistency ⇒ Aspera::PersistencyFolder
-
#query_read_delete(default: nil) ⇒ Hash?
Query parameters in URL suitable for REST: list/‘GET` and delete/`DELETE`.
- #transfer ⇒ Aspera::Cli::TransferAgent
-
#value_create_modify(command:, description: nil, type: Hash, bulk: false, default: nil, schema: nil) ⇒ Hash+
Retrieves an extended value from command line.
Constructor Details
#initialize(context:) ⇒ Base
Returns a new instance of Base.
29 30 31 32 33 34 35 |
# File 'lib/aspera/cli/plugins/base.rb', line 29 def initialize(context:) # Check presence in descendant of mandatory method and constant Aspera.assert(respond_to?(:execute_action), type: InternalError){"Missing method 'execute_action' in #{self.class}"} Aspera.assert(self.class.const_defined?(:ACTIONS), type: InternalError){"Missing constant 'ACTIONS' in #{self.class}"} @context = context add_manual_header if @context.man_header end |
Instance Attribute Details
#context ⇒ Object (readonly)
Global objects
38 39 40 |
# File 'lib/aspera/cli/plugins/base.rb', line 38 def context @context end |
Class Method Details
.declare_options(options) ⇒ Object
22 23 24 25 26 |
# File 'lib/aspera/cli/plugins/base.rb', line 22 def () .declare(:query, 'Additional filter for for some commands (list/delete)', allowed: [Hash, Array, NilClass]) .declare(:bulk, 'Bulk operation (only some)', allowed: Allowed::TYPES_BOOLEAN, default: false) .declare(:bfail, 'Bulk operation error handling', allowed: Allowed::TYPES_BOOLEAN, default: true) end |
Instance Method Details
#add_manual_header(has_options = true) ⇒ Object
51 52 53 54 55 56 57 |
# File 'lib/aspera/cli/plugins/base.rb', line 51 def add_manual_header( = true) # Manual header for all plugins .parser.separator('') .parser.separator("COMMAND: #{self.class.name.split('::').last.downcase}") .parser.separator("SUBCOMMANDS: #{self.class.const_get(:ACTIONS).map(&:to_s).sort.join(' ')}") .parser.separator('OPTIONS:') if end |
#config ⇒ Aspera::Cli::Plugins::Config
45 |
# File 'lib/aspera/cli/plugins/base.rb', line 45 def config; @context.config; end |
#do_bulk_operation(command:, descr: nil, values: Hash, id_result: 'id', fields: :default, schema: nil) {|param| ... } ⇒ Hash
For create and delete operations: execute one action or multiple if bulk is yes
69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 |
# File 'lib/aspera/cli/plugins/base.rb', line 69 def do_bulk_operation(command:, descr: nil, values: Hash, id_result: 'id', fields: :default, schema: nil, &block) Aspera.assert(block_given?){'missing block'} is_bulk = .get_option(:bulk) case values when :identifier values = .instance_identifier(description: descr) when Class values = value_create_modify(command: command, description: descr, type: values, bulk: is_bulk, schema: schema) end # If not bulk, there is a single value params = is_bulk ? values : [values] Log.log.warn('Empty list given for bulk operation') if params.empty? Log.dump(:bulk_operation, params) result_list = [] params.each do |param| # Init for delete result = {id_result => param} begin # Execute custom code res = yield(param) # If block returns a hash, let's use this (create) result = res if res.is_a?(Hash) # TODO: remove when faspio gw api fixes this result = res.first if res.is_a?(Array) && res.first.is_a?(Hash) # Create -> created result['status'] = "#{command}#{'e' unless command.to_s.end_with?('e')}d".gsub(/yed$/, 'ied') rescue StandardError => e raise e if .get_option(:bfail) result['status'] = e.to_s end result_list.push(result) end display_fields = [id_result, 'status'] if is_bulk return Main.result_object_list(result_list, fields: display_fields) else display_fields = fields unless fields.eql?(:default) return Main.result_single_object(result_list.first, fields: display_fields) end end |
#entity_execute(api:, entity:, command: nil, display_fields: nil, items_key: nil, delete_style: nil, id_as_arg: false, is_singleton: false, list_query: nil) {|value| ... } ⇒ Hash
Operations: Create, Delete, Show, List, Modify
123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 |
# File 'lib/aspera/cli/plugins/base.rb', line 123 def entity_execute( api:, entity:, command: nil, display_fields: nil, items_key: nil, delete_style: nil, id_as_arg: false, is_singleton: false, list_query: nil, &block ) command = .get_next_command(Operations::ALL) if command.nil? if is_singleton one_res_path = entity elsif Operations::INSTANCE.include?(command) one_res_id = .instance_identifier(&block) one_res_path = "#{entity}/#{one_res_id}" one_res_path = "#{entity}?#{id_as_arg}=#{one_res_id}" if id_as_arg end case command when :create raise BadArgument, 'cannot create singleton' if is_singleton return do_bulk_operation(command: command, descr: 'data', fields: display_fields) do |params| api.create(entity, params) end when :delete raise BadArgument, 'cannot delete singleton' if is_singleton if !delete_style.nil? one_res_id = [one_res_id] unless one_res_id.is_a?(Array) Aspera.assert_type(one_res_id, Array, type: Cli::BadArgument) api.delete( entity, nil, content_type: Mime::JSON, body: {delete_style => one_res_id} ) return Main.result_status('deleted') end return do_bulk_operation(command: command, values: one_res_id) do |one_id| api.delete("#{entity}/#{one_id}", query_read_delete) {'id' => one_id} end when :show return Main.result_single_object(api.read(one_res_path), fields: display_fields) when :list data, http = api.read(entity, query_read_delete, ret: :both) return Main.result_empty if http.code == '204' # TODO: not generic : which application is this for ? if http['Content-Type'].start_with?('application/vnd.api+json') Log.log.debug('is vnd.api') data = data[entity] end data = data[items_key] if items_key case data when Hash return Main.result_single_object(data, fields: display_fields) when Array return Main.result_object_list(data, fields: display_fields) if data.empty? || data.first.is_a?(Hash) return Main.result_value_list(data) else Aspera.error_unexpected_value(data.class.name){'list type'} end when :modify parameters = value_create_modify(command: command) api.update(one_res_path, parameters) return Main.result_status('modified') else Aspera.error_unexpected_value(command){'command'} end end |
#formatter ⇒ Aspera::Cli::Formatter
47 |
# File 'lib/aspera/cli/plugins/base.rb', line 47 def formatter; @context.formatter; end |
#options ⇒ Aspera::Cli::Manager
41 |
# File 'lib/aspera/cli/plugins/base.rb', line 41 def ; @context.; end |
#persistency ⇒ Aspera::PersistencyFolder
49 |
# File 'lib/aspera/cli/plugins/base.rb', line 49 def persistency; @context.persistency; end |
#query_read_delete(default: nil) ⇒ Hash?
Query parameters in URL suitable for REST: list/‘GET` and delete/`DELETE`
199 200 201 202 203 204 205 206 207 208 209 210 |
# File 'lib/aspera/cli/plugins/base.rb', line 199 def query_read_delete(default: nil) # Dup default, as it could be frozen query = .get_option(:query) || default.dup Log.log.debug{"query_read_delete=#{query}".bg_red} begin # Check it is suitable URI.encode_www_form(query) unless query.nil? rescue StandardError => e raise Cli::BadArgument, "Query must be an extended value (Hash, Array) which can be encoded with URI.encode_www_form. Refer to manual. (#{e.})" end return query end |
#transfer ⇒ Aspera::Cli::TransferAgent
43 |
# File 'lib/aspera/cli/plugins/base.rb', line 43 def transfer; @context.transfer; end |
#value_create_modify(command:, description: nil, type: Hash, bulk: false, default: nil, schema: nil) ⇒ Hash+
Retrieves an extended value from command line. Used for creation or modification of entities.
221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 |
# File 'lib/aspera/cli/plugins/base.rb', line 221 def value_create_modify(command:, description: nil, type: Hash, bulk: false, default: nil, schema: nil) value = .get_next_argument( "parameters for #{command}#{" (#{description})" unless description.nil?}", mandatory: default.nil?, validation: bulk ? Array : type, schema: schema ) value = default if value.nil? unless type.nil? Aspera.assert_type(type, Class){'type'} if bulk Aspera.assert_array_all(value, type, type: Cli::BadArgument){'type'} else Aspera.assert_type(value, type, type: Cli::BadArgument){'type'} end end return value end |